/** \file beep-usage.h
 * \brief interface to beep usage message string
 * \author Copyright (C) 2018-2019 Hans Ulrich Niedermann
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *
 * \defgroup beep_usage beep(1) usage message
 *
 * @{
 *
 */

#ifndef BEEP_USAGE_H
#define BEEP_USAGE_H


/** Multi-line zero-terminated string containing the usage text for
 *  beep(1), as is contained in the beep-usage.txt text file.
 */
extern char beep_usage[];


#endif /* !defined(BEEP_USAGE_H) */


/** @} */


/*
 * Local Variables:
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
