/* Auto-generated from `beep-usage.txt`. Modify that file instead. */
#include "beep-usage.h"
char beep_usage[] =
  "Usage:\n"
  "  beep [--verbose|--debug] [-e DEVICE]\n"
  "       [-f FREQ_Hz] [-l LEN_ms] [-r REPS] [<-d|-D> DELAY_ms] [-s] [-c]\n"
  "  beep [OPTIONS] <TONE_OPTS> [-n|--new TONE_OPTS...]\n"
  "  beep [-h|--help]\n"
  "  beep [-v|-V|--version]\n"
  "\n"
  "Beep the PC speaker in a number of ways.\n"
  "\n"
  "Options:\n"
  "  Information options:\n"
  "    -h, --help\n"
  "                  print this usage information and exit\n"
  "    -V, -v, --version\n"
  "                  print program version information and exit\n"
  "\n"
  "  Global options:\n"
  "    -e, --device=DEVICE\n"
  "                  set the device to output the beeps to (see beep(1) man page)\n"
  "    --debug, --verbose\n"
  "                  make program output more verbose\n"
  "\n"
  "  Tone options:\n"
  "    -f FREQ_Hz    frequency of the tone in Hertz (Hz) (default: 440)\n"
  "    -l LENGTH_ms  length of the tone in milliseconds (ms) (default: 200)\n"
  "    -d DELAY_ms   delay between repetitions of the tone *without* delay after last\n"
  "                  repetition of the tone (default: 100)\n"
  "    -D DELAY_ms   delay between repetitions of the tone *with* delay after last\n"
  "                  repetition of the tone (default: no delay after last note)\n"
  "    -r REPS       number of repetitions of the last tone\n"
  "                  (default: 1 note without repeats)\n"
  "\n"
  "    -n, --new     start a new tone\n"
  "\n"
  "    -s            read text from stdin and write it through to stdout,\n"
  "                  beeping the last defined tone for every newline in the text,\n"
  "                  until EOF in stdin\n"
  "    -c            like -s, but beep for every character in the text\n"
  "\n"
  "Exit status:\n"
  "    0             if OK\n"
  "    non-0         if any problems occurred\n"
  "\n"
  "More documentation is available in the beep(1) man page via:  man 1 beep\n"
  ;
